#ifndef INPUTGATHERSERVER_H
#define INPUTGATHERSERVER_H

#include "event.h"

#include <QObject>
#include <QLocalServer>
#include <QLocalSocket>
#include <QVector>
#include <QMap>
#include <QMutex>

#define INPUT_GATHER_SERVER_NAME "InputGatherServer"

/*!
 * \brief 负责将事件向客户端进行分发
 * \details 基于QLocalServer QLocalSocket实现
 */

class LocalServer : public QObject
{
    Q_OBJECT
public:
    static LocalServer* self(QObject *parent = nullptr);
    ~LocalServer();

    void runServer();
    void send(Event event);
    void send(QByteArray msg);

private slots:
    void onNewConnectionHandler();
    void onSocketClose();
    void onSocketError(QLocalSocket::LocalSocketError);

private:
    LocalServer(QObject *parent = nullptr);
    void init();
    void initConnect();
    QString error2Str(QLocalSocket::LocalSocketError error);

    static LocalServer* _self;
    QLocalServer *m_localServer;
    QVector<QLocalSocket*> m_clientSockets;
    QMap<QMetaObject::Connection, QLocalSocket*> m_connectMapSocket;
};

#endif // INPUTGATHERSERVER_H
